﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using HLWebOKBAdminPlugin.ViewModels;
using System.Web.Security;
using System.ServiceModel.Security;
using HLWebPluginHost.OKBService;

namespace HLWebOKBAdminPlugin.Controllers
{
    public class AlgorithmClassController : Controller
    {
        //BASICS: controllers respond to input from the browser, decide what to do and return
        //        response to the user (Views)

        // for edit form 2 controller actions are needed
        // first controller action if form is first loaded to display algorithm class data
        // second controller handles the post action

        //returns a certificated service client for current user .... call this function in every method
        private OKBService.OKBServiceClient CertificateValidator() {
            OKBService.OKBServiceClient sc = new OKBService.OKBServiceClient();

            sc.ClientCredentials.UserName.UserName = Membership.GetUser().UserName;
            sc.ClientCredentials.UserName.Password = HttpContext.Session["pwd"].ToString();
            sc.ClientCredentials.ServiceCertificate.Authentication.CertificateValidationMode = X509CertificateValidationMode.None;

            return sc;
        }//CertificateValidator        

        //
        // GET: /AlgorithmClass/

        public ActionResult Index() {
            //the index action is the homepage of the site (AlgorithmClass)  
            //what happens: generate list of AlgorithmClass and commit the list to the index view

            //get certificated service client
            OKBService.OKBServiceClient sc = CertificateValidator();

            //get list of Algorithm Classes
            var algorithmClasses = new List<AlgorithmClass>();
            AlgorithmClass[] ac = sc.GetAlgorithmClasses();
            foreach (AlgorithmClass a in ac) {
                algorithmClasses.Add(a);
            }//foreach            

            //return Index View that contains a list of all AlgorithmClasses (Views/Index.aspx)
            return View(algorithmClasses);
        }//Index

        //
        // GET: /AlgorithmClass/Details/5

        public ActionResult Details(int id) {
            //what happens: get AlgorithmClass object , create a viewmodel which contains the data
            //              for the view and commit the viewmodel to the view
            //              => correlates to a HTTP GET request

            //get certificated service client
            OKBService.OKBServiceClient sc = CertificateValidator();

            //the viewmodel contains data for the view
            var viewModel = new AlgorithmClassViewModel {
                AlgorithmClass = sc.GetAlgorithmClass(id)
            };

            return View(viewModel);
        }//Details

        //
        // GET: /AlgorithmClass/Create

        public ActionResult Create() {
            //what happens: create empty AlgorithmClass object , create a viewmodel which contains the data
            //              for the view and commit the viewmodel to the view
            //              this Create action is needed for first time call of the page to display content

            //get certificated service client
            OKBService.OKBServiceClient sc = CertificateValidator();

            //the viewmodel with empty AlgorithmClass object
            var viewModel = new AlgorithmClassViewModel {
                AlgorithmClass = new AlgorithmClass()
            };

            return View(viewModel);
        }//Create

        //
        // POST: /AlgorithmClass/Create

        [HttpPost]
        public ActionResult Create(AlgorithmClass algorithmClass) {
            //what happens: the 2nd Create controller handles the POST action from the form
            //              an AlgorithmClass object is passed from the Create view by submit

            try {
                //get certificated service client
                OKBService.OKBServiceClient sc = CertificateValidator();

                //create new AlgorithmClass object
                AlgorithmClass ac = new HLWebPluginHost.OKBService.AlgorithmClass();
                ac.Id = algorithmClass.Id;
                ac.Name = algorithmClass.Name;
                ac.Description = algorithmClass.Description;

                //add new AlgorithmClass
                sc.AddAlgorithmClass(ac);

                //redirect to the home page
                return RedirectToAction("Index");
            }
            catch {
                //invalid - redisplay with errors
                var viewModel = new AlgorithmClassViewModel {
                    AlgorithmClass = algorithmClass
                };

                return View(viewModel);
            }
        }//Create - post

        //
        // GET: /AlgorithmClass/Edit/5

        public ActionResult Edit(int id) {
            //what happens: get AlgorithmClass object , create a viewmodel which contains the data
            //              for the view and commit the viewmodel to the view
            //              => correlates to a HTTP GET request
            
            //get certificated service client
            OKBService.OKBServiceClient sc = CertificateValidator();

            //the viewmodel contains data for the view
            var viewModel = new AlgorithmClassViewModel {
                AlgorithmClass = sc.GetAlgorithmClass(id)
            };
            return View(viewModel);
        }//Edit

        //
        // POST: /AlgorithmClass/Edit/5

        [HttpPost]
        public ActionResult Edit(int id, FormCollection collection) {
            //what happens: the 2nd Edit controller handles the POST action from the form
            //              an AlgorithmClass object is passed from the Edit view by submit

            //get certificated service client
            OKBService.OKBServiceClient sc = CertificateValidator();

            //get AlgorithmClass object from DB
            var algorithmClass = sc.GetAlgorithmClass(id);

            try {
                // update AlgorithmClass
                UpdateModel(algorithmClass, "AlgorithmClass");

                AlgorithmClass ac = new HLWebPluginHost.OKBService.AlgorithmClass();
                ac.Id = algorithmClass.Id;
                ac.Name = algorithmClass.Name;
                ac.Description = algorithmClass.Description;

                sc.UpdateAlgorithmClass(ac);

                return RedirectToAction("Index");
            }
            catch {
                var viewModel = new AlgorithmClassViewModel {
                    AlgorithmClass = algorithmClass
                };

                return View(viewModel);
            }
        }//Edit

        //
        // GET: /AlgorithmClass/Delete/5

        public ActionResult Delete(int id) {
            //what happens: get AlgorithmClass object , create a viewmodel which contains the data
            //              for the view and commit the viewmodel to the view
            //              => correlates to a HTTP GET request

            //get certificated service client
            OKBService.OKBServiceClient sc = CertificateValidator();

            //the viewmodel contains data for the view
            var viewModel = new AlgorithmClassViewModel {
                AlgorithmClass = sc.GetAlgorithmClass(id)
            };
            return View(viewModel);
        }//Delete

        //
        // POST: /AlgorithmClass/Delete/5

        [HttpPost]
        public ActionResult Delete(int id, FormCollection collection) {
            //what happens: the 2nd Delete controller handles the POST action from the form
            //              an AlgorithmClass object is passed from the Delete view by submit
   
            //get certificated service client
            OKBService.OKBServiceClient sc = CertificateValidator();

            //load object from db
            var algorithmClass = sc.GetAlgorithmClass(id);

            try {
                //delete object
                sc.DeleteAlgorithmClass(id);

                return RedirectToAction("Index");
            }
            catch {
                var viewModel = new AlgorithmClassViewModel {
                    AlgorithmClass = algorithmClass
                };

                return View(viewModel);
            }
        }//Delete

    }
}
